#include <ch.h>
#include <hal.h>
#include <chprintf.h>

#include "board.h"
#include "usbcfg.h"

/**
 * blinking thread to show that we are alive
 */
static WORKING_AREA(blinkingThreadStack, 128);

static void blinkingThread(void *arg) {
	chRegSetThreadName("blinking thread");

	while (TRUE) {
		palSetPad(GPIOD, GPIOD_LED5); // red - blinks via thread
		chThdSleepMilliseconds(100);
		palClearPad(GPIOD, GPIOD_LED5); // red - blinks via thread
		chThdSleepMilliseconds(100);
	}
}

/* Virtual serial port over USB.*/
SerialUSBDriver SDU1;

void usb_serial_start(void) {
	/*
	 * Initializes a serial-over-USB CDC driver.
	 */
	sduObjectInit(&SDU1);
	sduStart(&SDU1, &serusbcfg);

	/*
	 * Activates the USB driver and then the USB bus pull-up on D+.
	 * Note, a delay is inserted in order to not have to disconnect the cable
	 * after a reset.
	 */
	usbDisconnectBus(serusbcfg.usbp);
	chThdSleepMilliseconds(1000);
	usbStart(serusbcfg.usbp, &usbcfg);
	usbConnectBus(serusbcfg.usbp);

	/*
	 * Activates the serial driver 2 using the driver default configuration.
	 * PA2(TX) and PA3(RX) are routed to USART2.
	 */
	sdStart(&SD2, NULL);
	palSetPadMode(GPIOA, 2, PAL_MODE_ALTERNATE(7));
	palSetPadMode(GPIOA, 3, PAL_MODE_ALTERNATE(7));

	while (TRUE) {
		if (SDU1.config->usbp->state == USB_ACTIVE) {
			break;
		}
		chThdSleepMilliseconds(500);
	}
}

#ifndef GPIO_AF_TIM2
#define GPIO_AF_TIM2 1
#endif

#define CRANK_DRIVER ICUD2
#define CRANK_INPUT_PORT GPIOA
#define CRANK_INPUT_PIN 5

#define TICKS_IN_MS  (CH_FREQUENCY / 1000)

volatile int rpm = 0;
int lastInputEventTime = -10 * CH_FREQUENCY;

void icuWidthCallback(ICUDriver *driver) {
	int now = chTimeNow();
	int diff = now - lastInputEventTime;
	rpm = 60000 * TICKS_IN_MS * 2 / 4 / diff;
	lastInputEventTime = now;
}

ICUConfig wave_icucfg = { ICU_INPUT_ACTIVE_LOW, 100000, icuWidthCallback, NULL };

int main(void) {
	halInit();
	chSysInit();

	// this thread would blink one of the LEDs, that would look cool
	chThdCreateStatic(blinkingThreadStack, sizeof(blinkingThreadStack), NORMALPRIO, blinkingThread, NULL);

	// serial-over-usb initialization
	usb_serial_start();

	// configure input signal pin
	palSetPadMode(CRANK_INPUT_PORT, CRANK_INPUT_PIN, PAL_MODE_ALTERNATE(GPIO_AF_TIM2));

	// start input capture - we will handle input events and calculate RPM based on the timestamps
	icuStart(&CRANK_DRIVER, &wave_icucfg);
	icuEnable(&CRANK_DRIVER);

	while (TRUE) {
		// RPM value is updated by the input event handler
		chprintf(&SDU1, "rpm: %d\r\n", rpm);
		chThdSleep(100);
	}

	return 0;
}

void print() {

}
